@@ -83,6 +83,18 @@ class Varnish(object):
         self.options.setdefault("cachehtml", "off")
         self.options.setdefault("enable-passthru", "off")
 
+        if not "pidfile" in self.options:
+            if 'run-directory' in self.buildout['buildout']:
+                self.options["pidfile"] = os.path.join(self.buildout['buildout']['run-directory'], self.name + ".pid")
+            else:
+                self.options["pidfile"] = os.path.join(self.buildout['buildout']['directory'], "var", self.name + ".pid")
+
+        if not "logpidfile" in self.options:
+            if 'run-directory' in self.buildout['buildout']:
+                self.options["logpidfile"] = os.path.join(self.buildout['buildout']['run-directory'], self.name + "log.pid")
+            else:
+                self.options["logpidfile"] = os.path.join(self.buildout['buildout']['directory'], "var", self.name + "log.pid")
+
         major, minor = self.determine_varnish_version()
         if major != '2':
             raise zc.buildout.UserError("Only version 2 of Varnish is supported")
@@ -115,8 +127,12 @@ class Varnish(object):
         self.options.created(location)
         self.add_runner()
         self.create_config()
+
         if "telnet" in self.options:
             self.add_varnishadm()
+            if "name" in self.options:
+                self.add_varnish_tool()
+
         if "varnishlog" in self.options:
             self.add_log()
         return self.options.created()
@@ -129,10 +145,7 @@ class Varnish(object):
         target=os.path.join(self.buildout["buildout"]["bin-directory"],self.name)
         f=open(target, "wt")
 
-        if 'run-directory' in self.buildout['buildout']:
-            pidfile = os.path.join(self.buildout['buildout']['run-directory'], self.name + ".pid")
-        else:
-            pidfile = os.path.join(self.buildout['buildout']['directory'], "var", self.name + ".pid")
+        pidfile = self.options["pidfile"]
 
         storage = os.path.join(self.buildout["buildout"]['directory'], 'var', self.name + ".storage")
         args = """
@@ -177,14 +190,28 @@ class Varnish(object):
         os.chmod(target, 0755)
         self.options.created(target)
 
+    def add_varnish_tool(self):
+        path = self.buildout["buildout"]["bin-directory"]
+        egg_paths = [
+            self.buildout["buildout"]["develop-eggs-directory"],
+            self.buildout["buildout"]["eggs-directory"],
+            ]
+
+        args = ",".join(["'%s'" % self.options[x] for x in ('name', 'telnet', 'config')])
+
+        ws = zc.buildout.easy_install.install(["isotoma.recipe.varnish"], 
+            self.buildout["buildout"]["eggs-directory"],
+            executable=sys.executable, 
+            path=egg_paths,
+            )
+        zc.buildout.easy_install.scripts([(self.name+"ctl", "isotoma.recipe.varnish.varnishtool", "run")], ws, sys.executable, path, arguments=args)
+        self.options.created(os.path.join(path, self.name+"ctl"))
+
     def add_log(self):
         target=os.path.join(self.buildout["buildout"]["bin-directory"],self.name + "log")
         f = open(target, "wt")
 
-        if 'run-directory' in self.buildout['buildout']:
-            pidfile = os.path.join(self.buildout['buildout']['run-directory'], self.name + "log.pid")
-        else:
-            pidfile = os.path.join(self.buildout['buildout']['directory'], "var", self.name + "log.pid")
+        pidfile = self.options["logpidfile"]
 
         daemon = self.options['varnishlog']
 
